@@ -57,7 +57,15 @@ typedef struct serf_request_t serf_request_t;
  */
 #define SERF_ERROR_RANGE 100
 
+/* This code is for when this is the last response on this connection:
+ * i.e. do not send any more requests on this connection or expect
+ * any more responses.
+ */
 #define SERF_ERROR_CLOSING (APR_OS_START_USERERR + SERF_ERROR_RANGE + 1)
+/* This code is for when the connection terminated before the request
+ * could be processed on the other side.
+ */
+#define SERF_ERROR_REQUEST_LOST (APR_OS_START_USERERR + SERF_ERROR_RANGE + 2)
 
 /**
  * Create a new context for serf operations.
@@ -710,7 +718,7 @@ SERF_DECLARE(void) serf_debug__bucket_alloc_check(serf_bucket_alloc_t *allocator
 /* Version info */
 #define SERF_MAJOR_VERSION 0
 #define SERF_MINOR_VERSION 1
-#define SERF_PATCH_VERSION 1
+#define SERF_PATCH_VERSION 2
 
 /**
  * Check at compile time if the Serf version is at least a certain
